# coding: utf-8
import toml
with open('pyproject.toml', 'r') as f:
    config = toml.load(f)
    
print(config['project']['version'])
version = config['project']['version'].split('.')
print(f'version={version}')
new_version = input("Enter new version name default= x.y.z where x=Major, y=Minor, z=patch/correctif?\n")
config['project']['version'] = new_version
print(f"New version={new_version}")
with open('pyproject.toml', 'w') as f:
    toml.dump(config, f)





import argparse
from pathlib import Path


def virvar(path: str, **kwargs: str) -> None:
    """virvar.

    Mofifies the `activate` script of a virtual environment to add environment variables.

    Args:
        path (str): The path to the virtual environment directory.
        kwargs (dict): Arbitrary keyword arguments representig the 
                    environment variables to add.

    Returns:
        None:

    Description:
        This function takes the path to a virtual environment (created with `venv`) and a list of
         environment variables as key-value pairs. It adds these variables to the `activate` script
         of the virtual environment, so they are set when the environment is activated and unset
         when the environment is deactivated.

    Usage:
        >>> virvar("venv", DEBUG="TRUE", COEF=2.56, INT=5)
        Environment validated: Unix-like virtual environment detected, and environment variables successfully configured.

        This adds the following lines to the `activate` script:
        # Self destruct!
        unset DEBUG
        unset COEF
        unset INT

        # set environment variables
        export DEBUG="TRUE"
        export COEF=2.56
        export INT=5
    """

    # Unset variables after this
    start_unset = "# Self destruct!"
    
    # build the path
    # Path for POSIX-compliant systems (e.g., Linux, macOS, BSD)
    bin_path_unix = Path(path) / "bin" / "activate"
    #  Path for WINDOWS-compliant systems 
    bin_path_windows = Path(path) / "Scripts" / "activate"

    if bin_path_unix.exists():
        bin_path = bin_path_unix
        print("Environment validated: Unix-like virtual environment detected, and environment variables successfully configured.")
    elif bin_path_windows.exists():
        bin_path = bin_path_windows
        print("Environment validated: Windows-like virtual environment detected, and environment variables successfully configured.")
    else:
        print("Error: Virtual environment not found. Unable to configure environment variables. Please ensure the specified path is correct.")
        raise FileNotFoundError("No such file or directory")


    # Build the export and the unset variables list
    export = "\n#set environment variables\n"
    unset = "\n        #unset environment variable\n"
    for key, value in kwargs.items():
        if type(value)==str:
            export += f'export {key.upper()}="{value}"\n'
        else:
            export += f'export {key.upper()}={value}\n'

        unset += f"        unset {key.upper()}\n"

    with bin_path.open("r") as file:
        lines = file.readlines()

    with bin_path.open("w") as file:
        for line in lines:
            file.write(line)
            if start_unset in line:
                file.write(unset + "\n")
                break
        remaining_lines = lines[lines.index(line) + 1:]
        file.writelines(remaining_lines)
        file.write(export)


def main():
    # Create parser
    parser = argparse.ArgumentParser()

    # Add arguments to the parser
    parser.add_argument(
        "path",
        help="Path to the virtual environment created with venv python command",
    )
    parser.add_argument(
            "env_var",
            nargs="+", # Accepts one or more pairs key = "value"
            help='Environment variables in the format KEY="value"'
            )

    #Parse the arguments
    args = parser.parse_args()

    # Transforms pairs key=value into a dictionary
    env_var = dict(arg.split("=", 1) for arg in args.env_var)

    virvar(args.path, **env_var)


if __name__=="__main__":
    main()



import pytest
import shutil
import sys
from pathlib import Path
from virvar.virvar import main, virvar

@pytest.fixture
def create_fake_env(tmp_path):
    """
    Fixture to create a fake virtual environment with an activate file.

    Args:
        tmp_path (Path): Temporary directory provided by pytest.

    Returns:Path
        Path: Path to the activate file of the virtual environment.
    """
    # Create the virtual environment path
    fake_env_path = tmp_path / "venv"

    # Path for POSIX-compliant systems (e.g., Linux, macOS, BSD)
    bin_path_unix = fake_env_path / "bin"
    bin_path_unix.mkdir(parents=True)

    #  Path for WINDOWS-compliant systems 
    bin_path_windows = fake_env_path / "Scripts"
    bin_path_windows.mkdir(parents=True)

    fake_message = """# Fake activate script
echo "Activating fake virtual environment"
# Self destruct!
"""

    # Create the activate files
    activate_file_unix = bin_path_unix / "activate"
    activate_file_unix.write_text(fake_message)

    activate_file_windows = bin_path_windows / "activate"
    activate_file_windows.write_text(fake_message)

    return activate_file_unix, activate_file_windows


def test_create_fake_env(create_fake_env):
    """
    Verify that the create_fake_env fixture creates a valid activate file
    for unix and windows environment.
    """
    for activate_file in create_fake_env:
        print(activate_file, activate_file.parent, activate_file.parent.parent)
        # Check that the activate file exists
        assert activate_file.exists(), f"The activate file for {activate_file} should exist after the fixture runs."
        
        # Check the initial content of the activate file
        content = activate_file.read_text()
        assert "# Fake activate script" in content, "The activate file in unix environment should contain the expected script header."
        assert "# Self destruct!" in content, "The activate file in unix environment should contain the self-destruct marker."


def test_virvar_adds_env_vars(create_fake_env):
    """
    Test that virvar adds environment variables to the activate file correctly
    for unix and windows environment.
    """

    # Example environment variables to add
    env_vars = {"TEST_VAR1": "test_value1", "TEST_VAR2": "test_value2"}

    for activate_file in create_fake_env:
        # Call the virvar function
        virvar(str(activate_file.parent.parent), **env_vars)


        # Read the updated activate file
        updated_content = activate_file.read_text()

        # Verify that export commands are added
        assert 'export TEST_VAR1="test_value1"' in updated_content, "TEST_VAR1 export is missing."
        assert 'export TEST_VAR2="test_value2"' in updated_content, "TEST_VAR2 export is missing."

        # Verify that unset commands are added in the correct place
        assert "unset TEST_VAR1" in updated_content, "TEST_VAR1 unset is missing."
        assert "unset TEST_VAR2" in updated_content, "TEST_VAR2 unset is missing."

        # Verify placement of unset commands
        self_destruct_index = updated_content.index("# Self destruct!")
        unset_index = updated_content.index("unset TEST_VAR1")
        assert unset_index > self_destruct_index, "Unset commands should come after the # Self destruct! marker."
        # Removes the fake virtual environment to ensure a complete reset before the next
        # iteration of the test loop for each environment (Unix and Windows).
        shutil.rmtree(activate_file.parent)


def test_main_no_arguments(monkeypatch, capsys):
    # Simule un appel sans argument 
    # argparse appelle par défaut sys.exit et lève une exception SystemExit
    monkeypatch.setattr(sys, "argv", ["virvar"])

    with pytest.raises(SystemExit) as excinfo: # Capture l'exception SystemExit 
        main()

    # Vérifie que le code de sortie est 2 (argparse le fait pour des erreurs)
    assert excinfo.value.code == 2 

    # Vérifie que le bon message d'erreur est affiché
    captured = capsys.readouterr()
    assert "the following arguments are required" in captured.err


def test_main_with_arguments(monkeypatch, capsys, create_fake_env):
    # Utilise la fixture pour créer un environement fictif pour environement unix
    # et windows
    for fake_activate_file in create_fake_env:
        fake_env_path = fake_activate_file.parent.parent # Récupére le chemin de l'environement
        # Simule un appel avec des arguments
        monkeypatch.setattr(sys, "argv", ["virvar", str(fake_env_path), "ENV_VAR=value"])

        # Vérifie que la fonction s'éxécute sans erreur et retourne le bon code
        assert main() == None

        # Capture et vérifie les sorties
        captured = capsys.readouterr()
        # Vérifie la sortie
        assert "Environment validated:" and "virtual environment detected, and environment variables successfully configured." in captured.out
        # Removes the fake virtual environment to ensure a complete reset before the next
        # iteration of the test loop for each environment (Unix and Windows).


def test_main_with_incorrect_path(monkeypatch, capsys, create_fake_env):

    # Simule un appel avec un mauvais chemin pour l'environment virtuel
    monkeypatch.setattr(sys, "argv", ["virvar", "path/to/incorrect_file", "ENV_VAR=value"])

    # Vérifie que FileNotFoundError est levée avec un message attendu
    with pytest.raises(FileNotFoundError) as excinfo:
        main()
    
    # Vérifie que le message d'erreur de l'exception
    assert "No such file or directory" in str(excinfo.value)

    # Capture et vérifie la sortie standard
    captured = capsys.readouterr()
    print(captured)
    assert "Error: Virtual environment not found. Unable to configure environment variables. Please ensure the specified path is correct." in captured.out


def test_variable_types_match(create_fake_env):
    # Récupérer le chemin du fichier activate depuis la fixture pour environement unix
    # et windows
    for activate_file in create_fake_env:
        fake_env_path = activate_file.parent.parent  # Revenir au chemin du dossier venv

        # Appeler la fonction virvar avec des variables de types différents
        virvar(
            str(fake_env_path),
            STRING_VAR="value",
            INT_VAR=123,
            FLOAT_VAR=45.67,
        )

        # Lire le contenu du fichier activate
        content = activate_file.read_text()

        # Extraire les variables définies dans le fichier
        variables = {
            line.split("=", 1)[0].replace("export ", "").strip(): eval(line.split("=", 1)[1].strip())
            for line in content.splitlines()
            if line.startswith("export ")
        }

        # Vérifier que les types correspondent
        assert variables["STRING_VAR"] == "value"
        assert variables["INT_VAR"] == 123
        assert variables["FLOAT_VAR"] == 45.67
        # Removes the fake virtual environment to ensure a complete reset before the next
        # iteration of the test loop for each environment (Unix and Windows).
        shutil.rmtree(activate_file.parent)



# coding: utf-8
# %load type_kwargs
def detect_type(**kwargs):
    print(type(kwargs))
    for k, v in kwargs.items():
        print(k, "est", type(k))
        print(v, "est", type(v))
        print()
        
